
#define ALLOCCONSOLE_IN_WINDOWS_H 1
/* #undef DEBUG -- XXX: defined in the VS project */
/* #undef HAVE_ARPA_INET_H */
#define HAVE_ATEXIT 1
/* #undef HAVE_COCOA_COCOA_H */
/* #undef HAVE_CONIO_H */
/* #undef HAVE_CURSES_H */
/* #undef HAVE_DLFCN_H */
/* #undef HAVE_ENDIAN_H */
#define HAVE_ERRNO_H 1
/* #undef HAVE_FLDLN2 */
/* #undef HAVE_FSIN_FCOS */
/* #undef HAVE_HAVE_FSIN_FCOS */
#define HAVE_GETENV 1
/* #undef HAVE_GETOPT_H */
/* #undef HAVE_GETOPT_LONG */
/* #undef HAVE_GETTIMEOFDAY */
/* #undef HAVE_GLUTCHECKLOOP */
/* #undef HAVE_GLUTCLOSEFUNC */
/* #undef HAVE_GL_GLUT_H */
/* #undef HAVE_GL_GL_H */
/* #undef HAVE_HTONS */
/* #undef HAVE_IMLIB2_H */
/* #undef HAVE_INTTYPES_H */
/* #undef HAVE_IOCTL */
/* #undef HAVE_JNI_H */
/* #undef HAVE_LOCALE_H */
#define HAVE_MEMORY_H 1
/* #undef HAVE_NCURSESW_NCURSES_H */
/* #undef HAVE_NCURSES_H */
/* #undef HAVE_NCURSES_NCURSES_H */
/* #undef HAVE_NETINET_IN_H */
/* #undef HAVE_OPENGL_GL_H */
#define HAVE_PUTENV 1
/* #undef HAVE_RESIZETERM */
/* #undef HAVE_RESIZE_TERM */
/* #undef HAVE_RUBY_H */
/* #undef HAVE_SIGNAL */
/* #undef HAVE_SIGNAL_H */
/* #undef HAVE_SLANG_H */
/* #undef HAVE_SLANG_SLANG_H */
#define HAVE_SLEEP 1
/* #undef HAVE_SLSMG_UTF8_ENABLE */
#define HAVE_SNPRINTF 1
#define HAVE_SPRINTF_S 1
#define HAVE_STDARG_H 1
#define HAVE_STDIO_H 1
/* #undef HAVE_STDINT_H */
#define HAVE_STDLIB_H 1
#define HAVE_STRCASECMP 1
#define HAVE_STRINGS_H 1
#define HAVE_STRING_H 1
/* #undef HAVE_SYS_IOCTL_H */
#define HAVE_SYS_SOCKET_H 1
#define HAVE_SYS_STAT_H 1
/* #undef HAVE_SYS_TIME_H */
#define HAVE_SYS_TYPES_H 1
/* #undef HAVE_TERMIOS_H */
/* #undef HAVE_UNISTD_H */
/* #undef HAVE_USLEEP */
/* #undef HAVE_VSNPRINTF */
#define HAVE_VSNPRINTF_S 1
#define HAVE_WINDOWS_H 1
#define HAVE_WINSOCK2_H 1
/* #undef HAVE_X11_XKBLIB_H */
/* #undef HAVE_ZLIB_H */
/* #undef LT_OBJDIR -- XXX: unneeded */
/* #undef NO_MINUS_C_MINUS_O */
#define PACKAGE_BUGREPORT ""
#define PACKAGE_NAME "libcaca"
#define PACKAGE_STRING ""
#define PACKAGE_TARNAME ""
#define PACKAGE_URL "http://caca.zoy.org/wiki/libcaca"
#define PACKAGE_VERSION "0.99.beta18"
/* #undef PROF */
/* #undef RETSIGTYPE */
/* #undef SCREENUPDATE_IN_PC_H */
#define STDC_HEADERS 1
/* #undef USE_COCOA */
/* #undef USE_CONIO */
/* #undef USE_GL */
/* #undef USE_IMLIB2 */
/* #undef USE_KERNEL */
/* #undef USE_NCURSES */
/* #undef USE_PLUGINS */
/* #undef USE_SLANG */
/* #undef USE_VGA */
#define USE_WIN32 1
/* #undef USE_X11 */
/* #undef const */
#ifndef __cplusplus
#define inline __inline
#endif
#define strcasecmp stricmp
#define snprintf _snprintf

