.fragment {
       font-family: monospace, fixed;
       font-size: 95%;
}
pre.fragment {
        border: 1px solid #CCCCCC;
        background-color: #f5f5f5;
        margin-top: 4px; 
        margin-bottom: 4px;
        margin-left: 2px;
        margin-right: 8px;
        padding-left: 6px;
        padding-right: 6px;
        padding-top: 4px;
        padding-bottom: 4px;
}

span.keyword       { color: #008000 }
span.keywordtype   { color: #604020 }
span.keywordflow   { color: #e08000 }
span.comment       { color: #800000 }
span.preprocessor  { color: #806020 }
span.stringliteral { color: #002080 }
span.charliteral   { color: #008080 }

/* Style for detailed member documentation */
.memtemplate {
  font-size: 80%;
  color: #606060;
  font-weight: normal;
}
.memnav {
  background-color: #e8eef2;
  border: 1px solid #84b0c7;
  text-align: center;
  margin: 2px;
  margin-right: 15px;
  padding: 2px;
}
.memitem {
  padding: 4px;
  /*background-color: #eef3f5;*/
  /*border-width: 1px;*/
  /*border-style: solid;*/
  /*border-color: #dedeee;*/
  /*-moz-border-radius: 8px 8px 8px 8px;*/
}
.memname {
  white-space: nowrap;
  font-weight: bold;
}
.memdoc{
  padding-left: 10px;
}
.memproto {
  background-color: #d5e1e8;
  width: 100%;
  border-width: 1px;
  border-style: solid;
  border-color: #84b0c7;
  font-weight: bold;
  -moz-border-radius: 8px 8px 8px 8px;
}
.paramkey {
  text-align: right;
}
.paramtype {
  white-space: nowrap;
}
.paramname {
  color: #602020;
  font-style: italic;
}
/* End Styling for detailed member documentation */

/* Remove those ugly <hr>s */
hr {
  display: none;
}

